Ext.namespace('dSS', 'dSS.data');

dSS.data.GroupStore = Ext.extend(Ext.data.Store, {
    constructor: function(config) {
        // create a record constructor for zone records
        var groupRecord = Ext.data.Record.create([
        {
            name: 'name'
        },
        {
            name: 'id'
        },
        {
            name: 'isValid'
        },
        {
            name: 'color',
            deaultValue: -1
        },
        {
            name: 'sortorder',
            defaultValue: 16 
        },
        {
            name: 'css'
        },
        {
            name: 'locked',
            defaultValue: false
        },
        {
            name: 'isAutomatic',
            defaultValue: false
        },
        {
            name: 'protection'
        },
        {
            name: 'direction',
            defaultValue: ""
        }
        ]);

        // a json reader to read the zone data
        var groupReader = new Ext.data.JsonReader({
            root: 'groups'
        },
        groupRecord);

        Ext.apply(this, {
            reader: groupReader,
            hasMultiSort: true,
            multiSortInfo: {
                sorters: [
                    {
                        field: 'sortorder',
                        direction: 'ASC'
                    },
                    {
                        field: 'name',
                        direction: 'ASC'
                    }
                ],
                direction: 'ASC'
            }
        });

        dSS.data.GroupStore.superclass.constructor.call(this, arguments);

        // hack to decouple postprocessing of the store from load/remove events,
        // otherwise selection of the data view gets screwed up
        this.on('load', function() {
            var fn = this.wrapper.createDelegate(this);
            fn.defer(1, this);
        }, this);

        this.on('remove', function() {
            var fn = this.wrapper.createDelegate(this);
            fn.defer(1, this);
        }, this);

        dSS.MessageBus.on('redrawgroupseparators', this.handleSeparators.createDelegate(this));

    },
    doSort: function() {
        this.sort([ { field: 'sortorder', direction: 'ASC' },
                    { field: 'name', direction: 'ASC' } ]);
    },
    wrapper: function() {
        dSS.MessageBus.fireEvent('redrawgroupseparators');
    },
    handleSeparators: function() {
        var store = dSS.data.Loader.getGroupStore();
        var autoClusters = store.find('isAutomatic', true);
        var userGroups = store.findBy(function(record) {
            if ((dSS.util.isUserGroup(record.get('id')) === true) &&
                (record.get('isAutomatic') === false)) {
                return true;
            }
            return false;
        });

        if (userGroups !== -1) {
            if (store.getById(-1) === undefined) {
                var rec =
                {
                    id: -1,
                    sortorder: 199,
                    color: -1,
                    name: '&mdash;&mdash;&mdash;&mdash;&mdash;&mdash;' +
                          '&mdash;&mdash;&mdash;&mdash;&mdash;'
                };
                store.loadData({ groups: [ rec ] }, true);
            }
        } else {
            var rec = store.getById(-1);
            if (rec !== undefined) {
                store.remove(rec);
                store.commitChanges();
                dSS.MessageBus.fireEvent('resetgroupselection');
            }
        }
        if (autoClusters !== -1) {
            if (store.getById(-2) === undefined) {
                var rec =
                {
                    id: -2,
                    sortorder: 399,
                    color: -1,
                    name: '&mdash;&mdash;&mdash;&mdash;&mdash;&mdash;' +
                          '&mdash;&mdash;&mdash;&mdash;&mdash;'
                };
                store.loadData({ groups: [ rec ] }, true);
            }
        } else {
            var rec = store.getById(-2);
            if (rec !== undefined) {
                store.remove(rec);
            }
        }

        store.sort([ { field: 'sortorder', direction: 'ASC' },
                     { field: 'name', direction: 'ASC' } ]);
    }
});

