Ext.namespace("dSS", "dSS.devices", "dSS.config", "dSS.gthelper", "Ext.ux");

// devClass
//  stop button will be shown depending on the device class
dSS.devices.DeviceSettingsPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.addEvents('loading');

        this.dSUID = config.dSUID;
        this.previousBladeAngle = -1;
        this.previousColor = { h: -1, s: -1, v: -1 };
        this.defaultColor = { h: 0, s: 0, v: 0 };

        this.maxValue = 0xff;
        var decPrecision = 0;

        if (config.matrix.shadeposition === true) {
            this.maxValue = 0xffff;
            decPrecision = 2;
        }

        this.matrix = config.matrix;
        this.virtualDevice = config.virtualDevice;

        this.DEFAULT_HEIGHT = 176;
        this.EXTRA_HEIGHT = 206;
        this.EXTRA_HEIGHT_OUTPUT_CHANNELS = 305;
        var initialHeight = this.DEFAULT_HEIGHT;
        if (this.matrix.shadebladeang === true) {
            initialHeight = this.EXTRA_HEIGHT;
        } else if (this.matrix.outputchannels === true) {
            initialHeight = this.EXTRA_HEIGHT_OUTPUT_CHANNELS;
        }

        if (config.overrideHeight !== undefined) {
            this.EXTRA_HEIGHT = config.overrideHeight;
            initialHeight = config.overrideHeight;
        }


        var xPos = dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth');
        var xPosOffset = 280;

        this.valueLabel = new Ext.form.Label({
            x: 10,
            y: 16,
            text: dSS._("Value") + ':'
        });

        this.valueField = new Ext.form.NumberField({
            x: xPos - xPosOffset,
            y: 12,
            width: 50,
            allowBlank: false,
            allowNegative: false,
            allowDecimals: (decPrecision !== 0),
            decimalPrecision: decPrecision,
            forceDecimalPrecision: true,
            disabled: (config.devClass === dSS.config.deviceClass.BL),
            value: 0,
            minValue: 0,
            maxValue: 100,
            listeners: {
                change: this.updateSlider.createDelegate(this)
            }
        });

        this.valueSlider = new Ext.Slider({
            height: 150,
            minValue: 0,
            increment: (decPrecision === 0) ? 1 : 0.001,
            maxValue: 100,
            vertical: true,
            disabled: (config.devClass === dSS.config.deviceClass.BL),
            decimalPrecision: decPrecision,
            plugins: new Ext.slider.Tip({
                getText: function(thumb) {
                    return String.format('{0}%', thumb.value);
                }
            }),
            listeners: {
                changeComplete: this.handleSliderChangedProxy.createDelegate(this)
            }
        });

        this.bladeAngleLabel = new Ext.form.Label({
            x: 10,
            y: 178,
            text: dSS._("Set blade angle") + ':'
        });

        this.bladeAngleField = new Ext.form.NumberField({
            x: xPos - xPosOffset,
            y: 174,
            width: 50,
            allowBlank: false,
            allowNegative: false,
            allowDecimals: false,
            value: 0,
            minValue: 0,
            maxValue: 100,
            listeners: {
                change: this.updateAngleSlider.createDelegate(this)
            }
        });

        this.bladeAngleUnit = new Ext.form.Label({
            x: xPos - (xPosOffset - 55),
            y: 178,
            text: '%'
        });

        this.angleSlider = new Ext.Slider({
            width: 160,
            minValue: 0,
            increment: 1,
            maxValue: 100,
            vertical: false,
            decimalPrecision: 1,
            plugins: new Ext.slider.Tip({
                getText: function(thumb) {
                    return String.format('{0}%', thumb.value);
                }
            }),
            listeners: {
                changeComplete: this.handleAngleSliderChanged.createDelegate(this)
            }
        });

        var localizePicker =
        {
            text:
            {
                title: dSS._("Drag markers to pick a color")
            },
            tooltips:
            {
                hue:
                {
                    radio: dSS._("Set to Hue color mode"),
                    textbox: dSS._("Enter a Hue value (0-360°)")
                },
                saturation:
                {
                    radio: dSS._("Set to Saturation color mode"),
                    textbox: dSS._("Enter a Saturation value (0-100%)")
                },
                value:
                {
                    radio: dSS._("Set to Value color mode"),
                    textbox: dSS._("Enter a Value value (0-100%)")
                }
            }
        };

        this.colorPicker = new Ext.ux.JPicker({
            x: 0,
            y: 0,
            border: false,
            localization: localizePicker
        });

        this.configLink = new Ext.Panel({
            x: 0,
            y: 0,
            height: initialHeight,
            border: false,
            layout: 'absolute',
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("Clicking the button will open the vDC " +
                                "configuration page in a new tab or window."),
                    x: 0,
                    y: 30
                },
                {
                    xtype: 'button',
                    x: ((dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth') - 34) / 2) - (260 / 2),
                    y: (initialHeight / 2) - (25 / 2),
                    minWidth: 260,
                    height: 25,
                    handler: this.openVdcConfigPage.createDelegate(this),
                    text: dSS._("Open vDC configuration")
                }
            ]
        });

        this.stopButton = new Ext.Button({
            x: xPos - xPosOffset,
            y: 140,
            xtype: 'button',
            text: dSS._("Stop"),
            width: 50,
            handler: this.handleStop.createDelegate(this)
        });

        this.maxButton = new Ext.Button({
            xtype: 'button',
            x: xPos - (xPosOffset - 102),
            y: 12,
            text: dSS._("Max"),
            tooltip: dSS._("Set maximum value"),
            width: 130,
            handler: this.handleMax.createDelegate(this)
        });

        this.minButton = new Ext.Button({
            x: xPos - (xPosOffset - 102),
            y: 140,
            xtype: 'button',
            text: dSS._("Min"),
            tooltip: dSS._("Set minimum value"),
            width: 130,
            handler: this.handleMin.createDelegate(this)
        });

        this.incButton = new Ext.Button({
            x: xPos - (xPosOffset - 102),
            y: 54,
            iconCls: 'buttonUpIcon',
            tooltip: dSS._("Increase value"),
            width: 130,
            handler: this.handleInc.createDelegate(this)
        });

        this.decButton = new Ext.Button({
            x: xPos - (xPosOffset - 102),
            y: 98,
            iconCls: 'buttonDownIcon',
            tooltip: dSS._("Decrease value"),
            width: 130,
            handler: this.handleDec.createDelegate(this)
        });

        this.forcedModeOnButton = new Ext.Button({
            x: xPos - (xPosOffset - 102),
            y: 12,
            text: dSS._("Forced mode on"),
            tooltip: dSS._("Forced mode on"),
            width: 130,
            hidden: (config.devClass !== dSS.config.deviceClass.BL),
            handler: this.handleForcedModeOn.createDelegate(this)
        });

        this.forcedModeOffButton = new Ext.Button({
            x: xPos - (xPosOffset - 102),
            y: 76,
            text: dSS._("Forced mode off"),
            tooltip: dSS._("Forced mode off"),
            width: 130,
            hidden: (config.devClass !== dSS.config.deviceClass.BL),
            handler: this.handleForcedModeOff.createDelegate(this)
        });

        this.noForcedModeButton = new Ext.Button({
            x: xPos - (xPosOffset - 102),
            y: 140,
            xtype: 'button',
            text: dSS._("Normal mode"),
            tooltip: dSS._("Normal mode"),
            width: 130,
            hidden: (config.devClass !== dSS.config.deviceClass.BL),
            handler: this.handleNoForcedMode.createDelegate(this)
        });

        this.opModeLabel = new Ext.form.Label({
            x: 10,
            y: 125,
            text: dSS._("Operation mode") + ':',
            hidden: (config.devClass !== dSS.config.deviceClass.BL)
        });

        this.opModeText = new Ext.form.Label({
            x: 10,
            y: 145,
            hidden: (config.devClass !== dSS.config.deviceClass.BL)
        });

        this.sliderView = new Ext.Panel({
            layout: 'absolute',
            border: false,
            items:
            [
                this.valueLabel,
                this.valueField,
                {
                    x: xPos - (xPosOffset - 55),
                    y: 16,
                    xtype: 'label',
                    text: '%'
                },
                {
                    x: xPos - (xPosOffset - 70),
                    y: 12,
                    xtype: 'panel',
                    border: false,
                    layout: 'form',
                    items: [ this.valueSlider ]
                },
                this.maxButton,
                this.incButton,
                this.decButton,
                this.stopButton,
                this.minButton,
                this.forcedModeOnButton,
                this.forcedModeOffButton,
                this.noForcedModeButton,
                this.opModeLabel,
                this.opModeText,
                this.bladeAngleLabel,
                this.bladeAngleField,
                this.bladeAngleUnit,
                {
                    x: xPos - (xPosOffset - 74),
                    y: 174,
                    xtype: 'panel',
                    border: false,
                    layout: 'form',
                    items: [ this.angleSlider ]
                }
            ]
        });

        this.dirtyValue = false;
        this.dirtyAngle = false;
        this.dirtyColor = false;

        Ext.apply(this, {
            layout: 'fit',
            padding: 5,
            bubbleEvents: [ 'updateshadow', 'valuechanged' ],
            height: initialHeight,
            items:
            [
                this.sliderView,
                this.colorPicker,
                this.configLink
            ]
        });

        this.setData(this.dSUID, config.devClass, config.present,
                     config.matrix, config.virtualDevice, config.device);

        dSS.devices.DeviceSettingsPanel.superclass.constructor.apply(this, arguments);

        this.addEvents('updateshadow');

        this.on('afterrender', function() {
                this.fireEvent('updateshadow', initialHeight);
            },
            this);

        this.on('beforedestroy', function() {
                this.killRequest();
                this.closed = true;
            },
            this);

        this.valueField.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    var nv = parseFloat(field.getValue());
                    var ov = parseFloat(this.valueSlider.getValue());
                    if (nv !== ov) {
                        this.updateSlider(field, nv);
                    }
                }
            },
            this);

        this.bladeAngleField.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    var nv = field.getValue();
                    this.updateAngleSlider(field, nv,
                                           this.angleSlider.getValue());
                }
            },
            this);
        this.colorPicker.on('jpickercolorchanged',
                this.handleColorChanged.createDelegate(this), this);
    },
    openVdcConfigPage: function() {
        if (this.deviceRecord) {
            dSS.util.openVdcLink(this.deviceRecord, 'settings');
        }
    },
    percentToValue: function(percent) {
        var value = percent / 100;
        value = value * this.maxValue;
        if (value < 0) {
            value = 0;
        } else if (value > this.maxValue) {
            value = this.maxValue;
        }
        return Math.round(value);
    },
    valueToPercent: function(value) {
        if (value < 0) {
            value = 0;
        }  else if (value > this.maxValue) {
            value = this.maxValue;
        }
        return (value / this.maxValue * 100);
    },
    handleSliderChangedProxy: function(slider, newVal, thumb) {
        this.fireEvent('valuechanged', this.dSUID);
        this.handleSliderChanged(slider, newVal, thumb);
    },
    handleSliderChanged: function(slider, newVal, thumb) {
        this.markAndAbortRead();
        this.valueField.setValue(newVal);
        if (this.maxValue <= 0xff) {
            dSS.util.setValueOnDevice(this.dSUID, this.percentToValue(newVal));
        } else {
            dSS.util.setOutputValueOnDevice(this.dSUID, 0x02,
                                            this.percentToValue(newVal));
        }
        if (this.matrix.shadebladeang === true) {
            if (newVal >= 90) {
                var oldAngle = this.bladeAngleField.getValue();
                this.bladeAngleField.setValue(100);
                this.updateAngleSlider.defer(300, this,
                        [ this.bladeAngleField, 100, oldAngle ]);
            }
        }
    },
    handleAngleSliderChanged: function(slider, newVal, thumb) {
        this.fireEvent('valuechanged', this.dSUID);
        this.markAndAbortRead();
        this.bladeAngleField.setValue(newVal);
        this.handleBladeAngleChanged(this.bladeAngleField, newVal,
                                     this.previousBladeAngle);
    },
    handleBladeAngleChanged: function(field, newVal, oldVal) {
        this.markAndAbortReadAngle();
        if (newVal !== this.previousBladeAngle) {
            dSS.util.genericRequest({
                    url: '/json/device/setConfig',
                    scope: this,
                    params:
                    {
                        'class': 64,
                        index: 0x04,
                        dsuid: this.dSUID,
                        value: Math.round(newVal / 100 * 0xff)
                    },
                    failureMsg: dSS._("Could not set device blade angle")
                });
            this.previousBladeAngle = newVal;
            this.dirtyAngle = false;
        }
    },
    updateAngleSlider: function(field, newVal, oldVal) {
        this.fireEvent('valuechanged', this.dSUID);
        this.markAndAbortReadOutputChannels();
        var ov = this.angleSlider.getValue();
        if ((newVal !== oldVal) && (newVal !== ov)) {
            this.angleSlider.setValue(newVal);
            this.handleBladeAngleChanged(field, newVal, oldVal);
        }
    },
    handleColorChanged: function(panel, color) {
        this.markAndAbortReadOutputChannels();
        if (this.colorPicker.equalHSVColors(this.previousColor, color)) {
            return;
        }

        this.previousColor = color;
        this.dirtyColor = false;

        // hue usually goes from 0 to 360°, but we are limited to 358.6
        if (color.h > 358.6) {
            color.h = 358.6;
        }

        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
                channelvalues: 'hue=' + color.h +
                               ';saturation=' + color.s +
                               ';brightness=' + color.v,
                applyNow: true,
                dsuid: this.dSUID,
            },
            failureMsg: dSS._("Could not set device color")
        });
    },


    updateSlider: function(field, newVal) {
        this.fireEvent('valuechanged', this.dSUID);
        this.markAndAbortRead();
        var oldVal = this.valueSlider.getValue();
        if (parseFloat(newVal) !== parseFloat(oldVal)) {
            this.valueSlider.setValue(newVal);
            if (this.maxValue <= 0xff) {
                dSS.util.setValueOnDevice(this.dSUID,
                                          this.percentToValue(newVal));
            } else {
                dSS.util.setOutputValueOnDevice(this.dSUID, 0x02,
                                                this.percentToValue(newVal));
            }
            if (this.matrix.shadebladeang === true) {
                if (newVal >= 90) {
                    var oldAngle = this.bladeAngleField.getValue();
                    this.bladeAngleField.setValue(100);
                    this.updateAngleSlider.defer(300, this,
                            [ this.bladeAngleField, 100, oldAngle ]);
                }
            }
        }
    },
    handleMax: function() {
        this.fireEvent('valuechanged', this.dSUID);
        this.markAndAbortRead();
        dSS.util.callScene(this.dSUID, dSS.config.scenes.MAX);
        this.valueField.setValue(100);
        this.valueSlider.setValue(100);
        if (this.matrix.shadebladeang === true) {
            this.bladeAngleField.setValue(100);
            this.angleSlider.setValue(100);
            this.handleBladeAngleChanged.defer(300, this,
                    [ this.bladeAngleField, 100, this.previousBladeAngle ]);
        }
    },
    handleMin: function() {
        this.fireEvent('valuechanged', this.dSUID);
        this.markAndAbortRead();
        dSS.util.callScene(this.dSUID, dSS.config.scenes.MIN);
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        if (this.matrix.shadebladeang === true) {
            this.bladeAngleField.setValue(0);
            this.angleSlider.setValue(0);
            this.handleBladeAngleChanged.defer(300, this,
                    [ this.bladeAngleField, 0, this.previousBladeAngle ]);
        }
    },
    handleStop: function() {
        this.fireEvent('valuechanged', this.dSUID);
        this.markAndAbortRead();
        dSS.util.callScene(this.dSUID, dSS.config.scenes.STOP);
        if (this.matrix.shadeposition === true) {
            this.dirtyValue = false;
            this.readDeviceValue.defer(500, this, [ this.dSUID ]);
        }
    },
    handleInc: function() {
        this.fireEvent('valuechanged', this.dSUID);
        if (this.matrix.shadeposition === true) {
            dSS.util.callScene(this.dSUID, dSS.config.scenes.INC);
            this.dirtyValue = false;
            this.readDeviceValue.defer(500, this, [ this.dSUID ]);
        } else {
            var v = this.valueSlider.getValue() + 5;
            if (v > 100) {
                v = 100;
            }

            if (v != this.valueSlider.getValue()) {
                this.valueSlider.setValue(v);
                this.handleSliderChanged(this.valueSlider, v);
            }
        }
    },
    handleDec: function() {
        this.fireEvent('valuechanged', this.dSUID);
        this.markAndAbortRead();

        if (this.matrix.shadeposition === true) {
            dSS.util.callScene(this.dSUID, dSS.config.scenes.DEC);
            this.dirtyValue = false;
            this.readDeviceValue.defer(500, this, [ this.dSUID ]);
        } else {
            var v = this.valueSlider.getValue() - 5;
            if (v < 0) {
                v = 0;
            }

            if (v != this.valueSlider.getValue()) {
                this.valueSlider.setValue(v);
                this.handleSliderChanged(this.valueSlider, v);
            }
        }
    },
    handleForcedModeOn: function() {
        dSS.util.callScene(this.dSUID, dSS.config.scenes.LOCAL_ON); // 51
        this.readOperationMode.defer(500, this, [ this.dSUID ]);
    },
    handleForcedModeOff: function() {
        dSS.util.callScene(this.dSUID, dSS.config.scenes.LOCAL_OFF); // 50
        this.readOperationMode.defer(500, this, [ this.dSUID ]);
    },
    handleNoForcedMode: function() {
        dSS.util.callScene(this.dSUID, dSS.config.scenes.STOP); // 15
        this.readOperationMode.defer(500, this, [ this.dSUID ]);
    },
    clearData: function()
    {
        this.killRequest();
        this.disable();
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        this.bladeAngleField.setValue(0);
        this.angleSlider.setValue(0);
        this.colorPicker.setColor(this.defaultColor);

        this.dirtyValue = false;
        this.dirtyAngle = false;
        this.dirtyColor = false;
    },
    setData: function(dSUID, devClass, present, matrix, virtualDevice, record) {
        this.killRequest();
        this.dSUID = dSUID;
        this.matrix = matrix;
        this.configLink.setVisible(false);

        if ((matrix.shadeposition == true) && (matrix.outvalue8 === true)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Unsupported configuration"));
            present = false;
        }

        if (matrix.shadeposition === true) {
            this.maxValue = 0xffff;
        } else {
            this.maxValue = 0xff;
        }

        if (matrix.shadeposition === true) {
            this.valueLabel.setText(dSS._("Position") + ':');
        } else {
            this.valueLabel.setText(dSS._("Value") + ':');
        }

        this.bladeAngleLabel.setVisible(matrix.shadebladeang);
        this.bladeAngleUnit.setVisible(matrix.shadebladeang);
        this.bladeAngleField.setVisible(matrix.shadebladeang);
        this.angleSlider.setVisible(matrix.shadebladeang);

        this.colorPicker.setVisible(matrix.outputchannels);
        this.sliderView.setVisible(!matrix.outputchannels);

        if ((matrix.shadebladeang === true) && (this.notFirstCall === true)) {
            if (this.getHeight() < this.EXTRA_HEIGHT) {
                this.setHeight(this.EXTRA_HEIGHT);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT);
            }
        } else if ((matrix.outputchannels === true) &&
                   (this.notFirstCall === true)) {
            if (this.getHeight() < this.EXTRA_HEIGHT_OUTPUT_CHANNELS) {
                this.setHeight(this.EXTRA_HEIGHT_OUTPUT_CHANNELS);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT_OUTPUT_CHANNELS);
            }
        }


        if ((devClass === dSS.config.deviceClass.GR) ||
            (devClass === dSS.config.deviceClass.GN) ||
            (devClass === dSS.config.deviceClass.RT)) {
            this.stopButton.setVisible(true);
        } else {
            this.stopButton.setVisible(false);
        }

        if (devClass === dSS.config.deviceClass.GR) {
            this.minButton.setText(dSS._("Down"));
            this.maxButton.setText(dSS._("Up"));
            this.forcedModeOnButton.hide();
            this.forcedModeOffButton.hide();
            this.noForcedModeButton.hide();
            this.opModeLabel.hide();
            this.opModeText.hide();
            this.maxButton.show();
            this.minButton.show();
            this.decButton.show();
            this.incButton.show();
            this.valueField.enable();
            this.valueSlider.enable();
        } else if (devClass === dSS.config.deviceClass.BL) {
            this.maxButton.hide();
            this.minButton.hide();
            this.decButton.hide();
            this.incButton.hide();
            this.forcedModeOnButton.show();
            this.forcedModeOffButton.show();
            this.noForcedModeButton.show();
            if (virtualDevice === true) {
                this.forcedModeOnButton.disable();
                this.forcedModeOffButton.disable();
                this.noForcedModeButton.disable();
                this.opModeLabel.hide();
                this.opModeText.hide();
            } else {
                this.opModeLabel.show();
                this.opModeText.show();
                this.opModeText.setText(dSS._("loading") + '...');
            }
            this.valueField.disable();
            this.valueSlider.disable();
        } else {
            this.minButton.setText(dSS._("Min"));
            this.maxButton.setText(dSS._("Max"));
            this.forcedModeOnButton.hide();
            this.forcedModeOffButton.hide();
            this.noForcedModeButton.hide();
            this.opModeLabel.hide();
            this.opModeText.hide();
            this.maxButton.show();
            this.minButton.show();
            this.decButton.show();
            this.decButton.show();
            this.incButton.show();
            this.valueField.enable();
            this.valueSlider.enable();
        }
        this.doLayout();
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        this.bladeAngleField.setValue(0);
        this.angleSlider.setValue(0);
        this.colorPicker.setColor({ h: 0, s: 0, v: 0 });

        this.dirtyValue = false;
        this.dirtyAngle = false;
        this.dirtyColor = false;
        if ((present === true) && (matrix.outvalue8 || matrix.shadeposition ||
                                   matrix.outputchannels)) {
            this.enable();
            if (this.matrix.outputchannels === true) {
                this.readOutputChannelValues.defer(0, this, [this.dSUID]);
            } else {
                this.readDeviceValue.defer(0, this, [this.dSUID]);
            }
        } else {
            if (virtualDevice === true) {
                this.readConfigLink(this.dSUID);
                this.sliderView.setVisible(false);
                this.colorPicker.setVisible(false);
                this.configLink.setVisible(true);
                this.deviceRecord = record;
            } else {
                this.disable();
            }
        }
        this.notFirstCall = true;
    },
    killRequest: function() {
        this.killRequestValue();
        this.killRequestAngle();
        this.killRequestOutputChannels();
        this.killRequestOperationMode();
        this.killRequestConfigLink();
    },
    killRequestValue: function() {
        if (this.AjaxRequestValue !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestValue);
            this.AjaxRequestValue = undefined;
            this.readBladeAngleValue(this.dSUID);
        }
    },
    killRequestAngle: function() {
        if (this.AjaxRequestAngle !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestAngle);
            this.AjaxRequestAngle = undefined;
        }
    },
    killRequestOutputChannels: function() {
        if (this.AjaxRequestOutputChannels !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestOutputChannels);
            this.AjaxRequestOutputChannels = undefined;
        }
    },
    killRequestOperationMode: function() {
        if (this.AjaxRequestOperationMode !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestOperationMode);
            this.AjaxRequestOperationMode = undefined;
        }
    },
    killRequestConfigLink: function() {
        if (this.AjaxRequestConfigLink !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestConfigLink);
            this.AjaxRequestConfigLink = undefined;
        }
    },

    readDeviceValue: function(reqdSUID)
    {
        this.killRequest();

        if (reqdSUID !== this.dSUID) {
            return;
        }

        this.fireEvent('loading', true);

        var request = 'getConfigWord';
        var offset = 0x02;

        if (this.maxValue <= 0xff) {
            request = 'getConfig';
            offset = 0;
        }

        if (this.matrix.pwmvalue === true) {
            request = 'getConfig';
            offset = 4;
        }

        this.AjaxRequestValue = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsuid: reqdSUID
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = undefined;
                if ((this.closed === true) ||
                    (this.dirtyValue === true)  ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device value") +
                            ': ' + jsonData.message);
                        this.valueField.markInvalid(
                            dSS._("Could not load current device value") +
                            ': ' + jsonData.message);
                    } else if (!isNaN(jsonData.result.value)) {
                        var p;
                        if (this.matrix.pwmvalue === true) {
                            p = jsonData.result.value;
                        } else {
                            p = this.valueToPercent(jsonData.result.value);
                        }
                        this.valueSlider.setValue(p);
                        this.valueField.setValue(p);
                    }

                    if (this.matrix.shadebladeang === true) {
                        this.readBladeAngleValue(this.dSUID);
                    }
                    if (this.matrix.pwmvalue === true) {
                        this.readOperationMode(this.dSUID);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        err);
                    this.valueField.markInvalid(
                        dSS._("Could not load current device value") + ': ' +
                        err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        jsonData.message);

                    if (this.matrix.shadebladeang === true) {
                        this.readBladeAngleValue(this.dSUID);
                    }
                    if (this.matrix.pwmvalue === true) {
                        this.readOperationMode(this.dSUID);
                    }
                    this.valueField.markInvalid(
                        dSS._("Could not load current device value") + ': ' +
                        jsonData.message);
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        err);
                    this.valueField.markInvalid(
                        dSS._("Could not load current device value") + ': ' +
                        err);
                }
            }
        });
    },
    markAndAbortRead: function() {
        this.dirtyValue = true;
        this.fireEvent('loading', false);
        this.killRequestValue();
    },
    markAndAbortReadAngle: function() {
        this.dirtyAngle = true;
        this.fireEvent('loading', false);
        this.killRequestAngle();
    },
    markAndAbortReadOperationMode: function() {
        this.fireEvent('loading', false);
        this.killRequestOperationMode();
    },
    markAndAbortReadOutputChannels: function() {
        this.dirtyColor = true;
        this.fireEvent('loading', false);
        this.killRequestOutputChannels();
    },
    readBladeAngleValue: function(reqdSUID) {
        this.killRequestValue();

        if (reqdSUID !== this.dSUID) {
            return;
        }

        this.fireEvent('loading', true);
        var request = 'getConfig';
        var offset = 0x04;

        this.AjaxRequestAngle = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsuid: reqdSUID
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestAngle = undefined;
                if ((this.closed === true) ||
                    (this.dirtyAngle === true)  ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device blade angle position") +
                            ': ' + jsonData.message);
                        this.bladeAngleField.markInvalid(
                            dSS._("Could not load current device blade angle position") +
                            ': ' + jsonData.message);

                    } else if (!isNaN(jsonData.result.value)) {
                        this.previousBladeAngle = Math.round(jsonData.result.value * 100 / 0xff);
                        this.bladeAngleField.setValue(this.previousBladeAngle);
                        this.angleSlider.setValue(this.previousBladeAngle);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        err);
                    this.bladeAngleField.markInvalid(
                        dSS._("Could not load current device blade angle position") +
                        ': ' + err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestAngle = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        jsonData.message);
                    this.bladeAngleField.markInvalid(
                        dSS._("Could not load current device blade angle position") +
                        ': ' + jsonData.message);

                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        err);
                    this.bladeAngleField.markInvalid(
                        dSS._("Could not load current device blade angle position") +
                        ': ' + err);
                }
            }
        });
    },
    readOperationMode: function(reqdSUID) {
        this.killRequestOperationMode();

        if (reqdSUID !== this.dSUID) {
            return;
        }

        this.fireEvent('loading', true);
        this.opModeText.setText(dSS._("loading") + '...');
        var request = 'getConfig';
        var offset = 0x05;

        this.AjaxRequestAngle = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsuid: reqdSUID
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOperationMode = undefined;
                if ((this.closed === true) ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device operation mode") +
                            ': ' + jsonData.message);
                        this.opModeText.setText("");

                    } else if (!isNaN(jsonData.result.value)) {
                        var text = dSS.config.HeatingOperationModeStrings[jsonData.result.value];
                        this.opModeText.setText(dSS._(text));
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device operation mode") + ': ' +
                        err);
                    this.opModeText.setText("");
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOperationMode = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device operation mode") + ': ' +
                        jsonData.message);
                    this.opModeText.setText("");
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device operation mode") + ': ' +
                        err);
                    this.opModeText.setText("");
                }
            }
        });
    },

    readOutputChannelValues: function(reqdSUID) {
        this.killRequestOutputChannels();

        if (reqdSUID !== this.dSUID) {
            return;
        }

        this.fireEvent('loading', true);
        this.AjaxRequestOutputChannels = Ext.Ajax.request({
            url: '/json/device/getOutputChannelValue',
            method: 'GET',
            params:
            {
                dsuid: reqdSUID,
                channels: 'hue;saturation;brightness'
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOutputChannels = undefined;
                if ((this.closed === true) ||
                    (this.dirtyColor === true)  ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current output channel " +
                                  "values") + ': ' + jsonData.message);
                        return;
                    }

                    var hue;
                    var sat;
                    var val;
                    var i;
                    for (i = 0; i < jsonData.result.channels.length; i++) {
                        var chan = jsonData.result.channels[i];
                        if (chan.channel === "hue") {
                            hue = Math.round(chan.value * 10) / 10;
                            continue;
                        }
                        if (chan.channel === "saturation") {
                            sat = Math.round(chan.value * 10) / 10;
                            continue;
                        }
                        if (chan.channel === "brightness") {
                            val = Math.round(chan.value * 10) / 10;
                            continue;
                        }
                    }

                    if (!isNaN(hue) && !isNaN(sat) && !isNaN(val)) {
                        this.previousColor.h = hue;
                        this.previousColor.s = sat;
                        this.previousColor.v = val;

                        this.colorPicker.setColor(this.previousColor);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOutputChannels = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + jsonData.message);
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + err);
                }
            }
        });
    },

    readConfigLink: function(reqdSUID) {
        this.killRequestConfigLink();

        if (reqdSUID !== this.dSUID) {
            return;
        }

        this.fireEvent('loading', true);

        this.AjaxRequestConfigLink = Ext.Ajax.request({
            url: '/json/property/query',
            method: 'GET',
            params:
            {
                query: '/apartment/zones/zone0/devices/' + this.dSUID +
                       '/productInfo(ConfigLink,State,ProductState)'
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestConfigLink = undefined;
                if ((this.closed === true) ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if ((jsonData.ok !== true) ||
                        (jsonData.result == undefined) ||
                        (jsonData.result.productInfo == undefined) ||
                        (jsonData.result.productInfo.length == 0) ||
                        (jsonData.result.productInfo[0].State !== "Valid") ||
                        (jsonData.result.productInfo[0].ProductState !== "Valid")) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("The configuration page location is " +
                                  "unknown and cannot be loaded."));
                        this.disable();
                        return;
                    } else {
                        this.enable();
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("The configuration page location is " +
                              "unknown and cannot be loaded."));
                    this.disable();
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestConfigLink = undefined;
                if (this.closed === true) {
                    return;
                }

                this.disable();
            }
        });
    }
});

Ext.reg('dssdevsettingspanel', dSS.devices.DeviceSettingsPanel);

dSS.devices.DeviceSettingsWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        this.HEIGHT_OFFSET = 130;

        if ((config.devices === undefined) || (config.index === undefined) ||
            (config.devices.length === 0)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("No information for device"));
            return;
        }

        var mainPanel = new dSS.devices.DeviceSettingsRootPanel({
            devices: config.devices,
            index: config.index,
            rowSel: config.rowSel,
            border: false,
            maskElement: config.maskElement
        });

        Ext.apply(this, {
            layout: 'fit',
            padding: 10,
            resizable: false,
            height: 306,
            width: dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth'),
            title: dSS._("Device settings"),
            items: [ mainPanel ]
        });

        dSS.devices.DeviceSettingsWindow.superclass.constructor.apply(this,
                                                                     arguments);

        mainPanel.on('updatetitleicon', function(icon) {
            this.setIconClass(icon);
        },
        this);

        mainPanel.on('close', function() {
            this.close();
        },
        this);

        mainPanel.on('updateshadow', function(height) {
            this.setHeight(height + this.HEIGHT_OFFSET);
            var s = this.getEl().shadow;
            if (s) {
                s.realign(this.x, this.y, this.getWidth(), this.getHeight());
            }
        },
        this);

    }
});

// parameters:
//  maskElement
//  device record from the device store
dSS.devices.DeviceSettingsRootPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {

        if (config.rowSel !== undefined) {
            this.rowSel = config.rowSel;
        }

        var disableLeft = true;
        var disableRight = true;

        this.devices = config.devices;
        this.index = config.index;

        var device;
        if (this.devices.length > 0) {
            device = this.devices[this.index];
        }

        if (this.devices.length > 1) {
            if (this.index > 0) {
                disableLeft = false;
            }
            if (this.index < this.devices.length - 1) {
                disableRight = false;
            }
        }

        this.embedded = false;
        if (config.embedded === true) {
            this.embedded = true;
        }

        var devCls;
        if (device !== undefined) {
            var devInfo = dSS.util.parseProductID(device.get('productID'));
            devCls = dSS.util.parseFunctionID(device.get('functionID'));
            var matrix;
            try {
                matrix = dSS.devprops.GetVisibleProperties(device);
            } catch (error) {
                matrix = error;
            }
        } else {
            try {
                matrix = dSS.devprops.GetVisibleProperties();
            } catch (error) {
                matrix = error;
            }
        }

        this.left = new Ext.Button({
            iconCls: 'buttonLeftIcon',
            tooltip: dSS._("Select previous device"),
            width: 50,
            disabled: disableLeft,
            handler: this.previousDevice.createDelegate(this)
        });

        this.right = new Ext.Button({
            iconCls: 'buttonRightIcon',
            tooltip: dSS._("Select next device"),
            width: 50,
            disabled: disableRight,
            handler: this.nextDevice.createDelegate(this)
        });

        this.titleIcon = '';
        var enabled = false;
        var devName = " ";
        var devdSUID = "";
        var devVirtualDevice = false;
        var devDisplayID = "";
        var devPresent = false;
        var devClass = 0;

        if (device !== undefined) {
            this.titleIcon = 'titleIcon' + dSS.util.genCSSName(device);
            enabled = ((device.get('isPresent') === true) &&
                       (matrix.outvalue8 || matrix.shadeposition));
            devName = device.get('name');
            devdSUID = device.get('dsuid');
            devDisplayID = device.get('DisplayID');
            devPresent = device.get('isPresent');
            devVirtualDevice = (device.get('VdcModelUID') != '');
            devClass = devCls[0];

            // output mode dependency, hack for BL-KM
            if (devClass === dSS.config.deviceClass.BL) {
                var devType = dSS.config.deviceTypes[devInfo[0]];
                if ((devType === "KM") && (devInfo[1] === 200)) {
                    var hack = this.getHeatingMods(matrix,
                                                   device.get('outputMode'));
                    matrix = hack.matrix;
                    devClass = hack.deviceClass;
                }
            }
        }

        var buttons =
        [
            this.left,
            this.right,
            new Ext.Toolbar.Fill()
        ];

        var bodyCss;
        var items = [];
        if (this.embedded === false) {
            buttons.push({
                xtype: 'button',
                text: dSS._("Close"),
                scope: this,
                handler: function() {
                    if (this.maskElement !== undefined) {
                        this.maskElement.unmask();
                    }
                    this.fireEvent('close');
                }
            });
            var info =
            {
                xtype: 'panel',
                ref: 'info',
                border: false,
                items:
                [
                    dSS.util.CreateDefaultDeviceInfoPanel(devName, devDisplayID)
                ]
            };

            items.push(info);
        } else {
            bodyCss = 'x-panel-no-left-border x-panel-no-right-border x-panel-no-top-border';
        }

        items.push(
        {
            xtype: 'dssdevsettingspanel',
            ref: 'settings',
            bodyCssClass: bodyCss,
            dSUID: devdSUID,
            DisplayID: devDisplayID,
            devClass: devClass,
            present: devPresent,
            matrix: matrix,
            virtualDevice: devVirtualDevice,
            device: device
        });

        Ext.apply(this, {
            bubbleEvents: [ 'updatetitleicon', 'close', 'updateshadow',
                            'valuechanged' ],
            layout: 'table',
            bodyBorder: false,
            layoutConfig: {
                columns: 1,
                tableAttrs: {
                    width: '100%'
                }
            },
            items: items,
            buttonAlign: 'left',
            buttons: buttons
        });

        dSS.devices.DeviceSettingsRootPanel.superclass.constructor.apply(this,
                                                                     arguments);
        this.on('afterrender', function() {
            var icon = enabled ? 'loading-indicator-small' : this.titleIcon;
            this.fireEvent('updatetitleicon', icon);
        },
        this);

        this.addEvents('updateshadow', 'updatetitleicon', 'close');

        if (this.maskElement !== undefined) {
            this.maskElement.mask();
        }

        this.on('beforedestroy', function(component) {
            if (this.maskElement !== undefined) {
                this.maskElement.unmask();
            }
        },
        this);

        this.settings.on('loading', function(l) {
            var icon;
            if (l === true) {
                icon = 'loading-indicator-small';
            } else {
                icon = this.titleIcon;
            }
            this.fireEvent('updatetitleicon', icon);
        },
        this);

    },
    setupButtons: function() {
        if (this.index == 0) {
            this.left.disable();
        }

        if ((this.devices.length > 1) &&
            (this.index < (this.devices.length - 1))) {
            this.right.enable();
        }

        if (this.index === (this.devices.length - 1)) {
            this.right.disable();
        }

        if ((this.devices.length > 1) && (this.index > 0)) {
            this.left.enable();
        }

    },
    loadDevices: function(devices) {
        this.devices = devices;
        this.right.disable();
        this.left.disable();
        if (this.devices.length > 0) {
            this.index = 0;
            //this.setupButtons();
        }
    },
    setRowSelectionModel: function(model) {
        if (this.rowModel !== undefined) {
            this.rowModel.removeListener('rowselect',
                    this.loadRow.createDelegate(this), this);
        }
        this.rowSel = model;
        this.rowSel.addListener('rowselect',
                this.loadRow.createDelegate(this), this);
    },
    clearDialog: function(model, index, record) {
        if (this.ownerCt.isVisible()) {
            this.settings.clearData();
            if (!this.embedded) {
                this.info.removeAll(true);
                this.info.add(dSS.util.CreateDefaultDeviceInfoPanel(" ", " "));
                this.info.doLayout();
            }
        }
    },
    loadRow: function(model, index, record) {
        if (this.ownerCt.isVisible()) {
            if (record.get('isPresent') === false) {
                return;
            }
            if (this.index < this.devices.length) {
                this.index = index;
            }
            this.setupButtons();
            this.updateView(false);
        }
    },
    getHeatingMods: function(matrix, outputMode) {
        var cls = dSS.config.deviceClass.BL;
        var copy = Ext.util.JSON.decode(Ext.util.JSON.encode(matrix));
        switch (outputMode) {
            case dSS.config.deviceParams.OUTPUT_MODE_SWITCH:
            case dSS.config.deviceParams.OUTPUT_MODE_PWM:
                cls = dSS.config.deviceClass.GE; // hack
                copy.heatinggroup = false;
                copy.heatingoutmode = false;
                copy.pwmvalue = false;
                break;
            default:
                break;
        }
        return { "deviceClass": cls, "matrix": copy };
    },
    updateView: function(doSelect) {
        var device = this.devices[this.index];
        var devInfo = dSS.util.parseProductID(device.get('productID'));
        var devCls = dSS.util.parseFunctionID(device.get('functionID'));
        var matrix;
        try {
            matrix = dSS.devprops.GetVisibleProperties(device);
        } catch (error) {
            matrix = error;
        }

        this.titleIcon = 'titleIcon' + dSS.util.genCSSName(device);
        this.fireEvent('updatetitleicon', this.titleIcon);

        if (!this.embedded) {
            this.info.removeAll(true);
            this.info.add(
                dSS.util.CreateDefaultDeviceInfoPanel(device.get('name'),
                                                      device.get('DisplayID')));
            this.info.doLayout();
        }

        var devClass = devCls[0];
        // output mode dependency, hack for BL-KM
        if (devClass === dSS.config.deviceClass.BL) {
            var devType = dSS.config.deviceTypes[devInfo[0]];
            if ((devType === "KM") && (devInfo[1] === 200)) {
                var hack = this.getHeatingMods(matrix,
                                               device.get('outputMode'));
                matrix = hack.matrix;
                devClass = hack.deviceClass;
            }
        }
        this.settings.setData(device.get('dsuid'), devClass,
                              (device.get('isPresent') && (device.get('outputMode') != 0)),
                              matrix, device.get('isVdcDevice'), device);
        if ((this.rowSel !== undefined) && (doSelect !== false)) {
            this.rowSel.selectRow(this.index, false, false);
        }
    },
    previousDevice: function() {
        if (this.index > 0) {
            this.index--;
            if (this.index == 0) {
                this.left.disable();
            }

            if ((this.devices.length > 1) &&
                (this.index < (this.devices.length - 1))) {
                this.right.enable();
            }
        }
        this.updateView();
    },
    nextDevice: function() {
        if (this.index < (this.devices.length - 1)) {
            this.index++;
            if (this.index === (this.devices.length - 1)) {
                this.right.disable();
            }

            if ((this.devices.length > 1) && (this.index > 0)) {
                this.left.enable();
            }
        }
        this.updateView();
    }
});

Ext.reg('dssdevicesettings', dSS.devices.DeviceSettingsRootPanel);


dSS.devices.DeviceSceneSettingsPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.addEvents('loading');

        this.dSUID = config.dSUID;
        this.DisplayID = config.DisplayID;
        this.scene = config.scene;

        this.maxValue = 0xff;
        var decPrecision = 0;

        this.statusObject = {};
        this.statusObject.outputValue = 0;
        this.statusObject.bladeAngle = 0;

        if (config.matrix.shadeposition === true) {
            this.maxValue = 0xffff;
            decPrecision = 2;
        }

        this.dummy = false;
        if (config.dummy !== undefined) {
            this.dummy = config.dummy;
        }

        this.matrix = config.matrix;

        this.DEFAULT_HEIGHT = 176;
        this.EXTRA_HEIGHT = 206;
        var initialHeight = this.DEFAULT_HEIGHT;
        if (this.matrix.shadebladeang === true) {
            initialHeight = this.EXTRA_HEIGHT;
        }

        if (config.overrideHeight !== undefined) {
            this.EXTRA_HEIGHT = config.overrideHeight;
            initialHeight = config.overrideHeight;
        }

        var xPos = dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth');
        if (config.extraWidth !== undefined) {
            xPos = xPos + config.extraWidth;
        }

        this.valueLabel = new Ext.form.Label({
            x: 10,
            y: 16,
            disabled: this.dummy,
            text: dSS._("Value") + ':'
        });

        this.valueField = new Ext.form.NumberField({
            disabled: this.dummy,
            x: xPos - 200,
            y: 12,
            width: 50,
            allowBlank: false,
            allowNegative: false,
            allowDecimals: (decPrecision !== 0),
            decimalPrecision: decPrecision,
            forceDecimalPrecision: true,
            value: 0,
            minValue: 0,
            maxValue: 100,
            listeners: {
                change: this.updateSlider.createDelegate(this)
            }
        });

        this.bladeAngleLabel = new Ext.form.Label({
            x: 10,
            y: 178,
            disabled: this.dummy,
            text: dSS._("Set blade angle") + ':'
        });

        this.bladeAngleField = new Ext.form.NumberField({
            x: xPos - 200,
            y: 174,
            disabled: this.dummy,
            width: 50,
            allowBlank: false,
            allowNegative: false,
            allowDecimals: false,
            decimalPrecision: 1,
            value: 0,
            minValue: 0,
            maxValue: 100,
            listeners: {
                change: this.updateAngleSlider.createDelegate(this)
            }
        });

        this.bladeAngleUnit = new Ext.form.Label({
            x: xPos - 145,
            y: 178,
            text: '%'
        });

        this.angleSlider = new Ext.Slider({
            disabled: this.dummy,
            width: 78,
            minValue: 0,
            increment: 1,
            maxValue: 100,
            vertical: false,
            decimalPrecision: 1,
            plugins: new Ext.slider.Tip({
                getText: function(thumb) {
                    return String.format('{0}%', thumb.value);
                }
            }),
            listeners: {
                changeComplete: this.handleAngleSliderChanged.createDelegate(this)
            }
        });

        this.valueSlider = new Ext.Slider({
            disabled: this.dummy,
            height: 150,
            minValue: 0,
            increment: (decPrecision === 0) ? 1 : 0.001,
            maxValue: 100,
            vertical: true,
            decimalPrecision: decPrecision,
            plugins: new Ext.slider.Tip({
                getText: function(thumb) {
                    return String.format('{0}%', thumb.value);
                }
            }),
            listeners: {
                changeComplete: this.handleSliderChangedProxy.createDelegate(this)
            }
        });

        this.stopButton = new Ext.Button({
            x: xPos - 200,
            y: 140,
            disabled: this.dummy,
            text: dSS._("Stop"),
            width: 50,
            handler: this.handleStop.createDelegate(this)
        });

        this.maxButton = new Ext.Button({
            x: xPos - 98,
            y: 12,
            disabled: this.dummy,
            text: dSS._("Max"),
            tooltip: dSS._("Set maximum value"),
            width: 50,
            handler: this.handleMax.createDelegate(this)
        });

        this.minButton = new Ext.Button({
            x: xPos - 98,
            y: 140,
            disabled: this.dummy,
            text: dSS._("Min"),
            tooltip: dSS._("Set minimum value"),
            width: 50,
            handler: this.handleMin.createDelegate(this)
        });

        Ext.apply(this, {
            layout: 'absolute',
            padding: 5,
            bubbleEvents: [ 'updateshadow', 'formchanged' ],
            height: initialHeight,
            items:
            [
                this.valueLabel,
                this.valueField,
                {
                    x: xPos - 145,
                    y: 16,
                    disabled: this.dummy,
                    xtype: 'label',
                    text: '%'
                },
                {
                    x: xPos - 130,
                    y: 12,
                    xtype: 'panel',
                    border: false,
                    layout: 'form',
                    items: [ this.valueSlider ]
                },
                this.maxButton,
                {
                    x: xPos - 98,
                    y: 54,
                    disabled: this.dummy,
                    xtype: 'button',
                    iconCls: 'buttonUpIcon',
                    tooltip: dSS._("Increase value"),
                    width: 50,
                    handler: this.handleInc.createDelegate(this)
                },
                {
                    x: xPos - 98,
                    y: 98,
                    disabled: this.dummy,
                    xtype: 'button',
                    iconCls: 'buttonDownIcon',
                    tooltip: dSS._("Decrease value"),
                    width: 50,
                    handler: this.handleDec.createDelegate(this)
                },
                this.stopButton,
                this.minButton,
                this.bladeAngleLabel,
                this.bladeAngleField,
                this.bladeAngleUnit,
                {
                    x: xPos - 126,
                    y: 174,
                    xtype: 'panel',
                    border: false,
                    layout: 'form',
                    items: [ this.angleSlider ]
                }
            ]
        });

        this.setData(this.dSUID, config.devClass, config.present, config.matrix);

        dSS.devices.DeviceSceneSettingsPanel.superclass.constructor.apply(this, arguments);

        this.valueField.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    var nv = parseFloat(field.getValue());
                    var ov = parseFloat(this.valueSlider.getValue());
                    if (nv !== ov) {
                        this.updateSlider(field, nv);
                    }
                }
            },
            this);

        this.bladeAngleField.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    var nv = field.getValue();
                    this.updateAngleSlider(field, nv,
                                           this.angleSlider.getValue());
                }
            },
            this);
    },
    percentToValue: function(percent, max) {
        if (max === undefined) {
            max = this.maxValue;
        }
        var value = percent / 100;
        value = value * max;
        if (value < 0) {
            value = 0;
        } else if (value > max) {
            value = max;
        }
        return Math.round(value);
    },
    valueToPercent: function(value, max) {
        if (max === undefined) {
            max = this.maxValue;
        }

        if (value < 0) {
            value = 0;
        }  else if (value > max) {
            value = max;
        }
        return (value / max * 100);
    },
    handleSliderChangedProxy: function(slider, newVal, thumb) {
        this.fireEvent('formchanged');
        this.handleSliderChanged(slider, newVal, thumb);
    },
    handleSliderChanged: function(slider, newVal, thumb) {
        this.valueField.setValue(newVal);
        if (this.matrix.shadebladeang === true) {
            if (newVal >= 90) {
                var oldAngle = this.bladeAngleField.getValue();
                this.bladeAngleField.setValue(100);
                this.updateAngleSlider.defer(300, this,
                        [ this.bladeAngleField, 100, oldAngle ]);
            }
        }
    },
    handleAngleSliderChanged: function(slider, newVal, thumb) {
        this.fireEvent('formchanged');
        this.bladeAngleField.setValue(newVal);
    },
    updateAngleSlider: function(field, newVal, oldVal) {
        this.fireEvent('valuechanged', this.dSUID);
        var ov = this.angleSlider.getValue();
        if ((newVal !== oldVal) && (newVal !== ov)) {
            this.angleSlider.setValue(newVal);
        }
    },
    updateSlider: function(field, newVal) {
        this.fireEvent('formchanged');
        var oldVal = this.valueSlider.getValue();
        if (parseFloat(newVal) !== parseFloat(oldVal)) {
            this.valueSlider.setValue(newVal);
            if (this.matrix.shadebladeang === true) {
                if (newVal >= 90) {
                    var oldAngle = this.bladeAngleField.getValue();
                    this.bladeAngleField.setValue(100);
                    this.updateAngleSlider.defer(300, this,
                            [ this.bladeAngleField, 100, oldAngle ]);
                }
            }
        }
    },
    handleMax: function() {
        this.fireEvent('formchanged');
        this.valueField.setValue(100);
        this.valueSlider.setValue(100);
        if (this.matrix.shadebladeang === true) {
            this.bladeAngleField.setValue(100);
            this.angleSlider.setValue(100);
        }
    },
    handleMin: function() {
        this.fireEvent('formchanged');
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        if (this.matrix.shadebladeang === true) {
            this.bladeAngleField.setValue(0);
            this.angleSlider.setValue(0);
        }
    },
    handleStop: function() {
        dSS.util.callScene(this.dSUID, dSS.config.scenes.STOP);
    },
    handleInc: function() {
        this.fireEvent('formchanged');
        var v = this.valueSlider.getValue() + 5;
        if (v > 100) {
            v = 100;
        }

        if (v != this.valueSlider.getValue()) {
            this.valueSlider.setValue(v);
            this.handleSliderChanged(this.valueSlider, v);
        }
    },
    handleDec: function() {
        this.fireEvent('formchanged');

        var v = this.valueSlider.getValue() - 5;
        if (v < 0) {
            v = 0;
        }

        if (v != this.valueSlider.getValue()) {
            this.valueSlider.setValue(v);
            this.handleSliderChanged(this.valueSlider, v);
        }
    },
    setData: function(dSUID, devClass, present, matrix) {
        this.dSUID = dSUID;
        this.matrix = matrix;
        if ((matrix.shadeposition == true) && (matrix.outvalue8 === true)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Unsupported configuration"));
            present = false;
        }

        if (matrix.shadeposition === true) {
            this.maxValue = 0xffff;
        } else {
            this.maxValue = 0xff;
        }

        if (matrix.shadeposition === true) {
            this.valueLabel.setText(dSS._("Position") + ':');
        } else {
            this.valueLabel.setText(dSS._("Value") + ':');
        }

        this.bladeAngleLabel.setVisible(matrix.shadebladeang);
        this.bladeAngleUnit.setVisible(matrix.shadebladeang);
        this.bladeAngleField.setVisible(matrix.shadebladeang);
        this.angleSlider.setVisible(matrix.shadebladeang);
        if ((matrix.shadebladeang === true) && (this.notFirstCall === true)) {
            if (this.getHeight() !== this.EXTRA_HEIGHT) {
                this.setHeight(this.EXTRA_HEIGHT);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT);
            }
        }

        if ((devClass === dSS.config.deviceClass.GR) ||
            (devClass === dSS.config.deviceClass.GN) ||
            (devClass === dSS.config.deviceClass.RT)) {
            this.stopButton.setVisible(true);
        } else {
            this.stopButton.setVisible(false);
        }

        if (devClass === dSS.config.deviceClass.GR) {
            this.minButton.setText(dSS._("Down"));
            this.maxButton.setText(dSS._("Up"));
        } else {
            this.minButton.setText(dSS._("Min"));
            this.maxButton.setText(dSS._("Max"));
        }
        this.doLayout();
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        this.bladeAngleField.setValue(0);
        this.angleSlider.setValue(0);
        /*
        if ((present === true) && (matrix.outvalue8 || matrix.shadeposition)) {
            this.enable();
        } else {
            this.disable();
        }
        */
        this.notFirstCall = true;
    },
    setDeviceValue: function(value)
    {
        if (isNaN(value)) {
            return;
        }

        this.statusObject.outputValue = value;
        var p = this.valueToPercent(value);
        this.valueSlider.setValue(p);
        this.valueField.setValue(p);
    },
    setBladeAngle: function(angle) {
        if (isNaN(angle)) {
            return;
        }

        this.statusObject.bladeAngle = angle;
        var p = Math.round(this.valueToPercent(angle, 0xff));
        this.bladeAngleField.setValue(p);
        this.angleSlider.setValue(p);
    },
    writeValuesToDevice: function() {
        var val = this.valueField.getValue();
        if (this.maxValue <= 0xff) {
            dSS.util.setValueOnDevice(this.dSUID, this.percentToValue(val));
        } else {
            dSS.util.setOutputValueOnDevice(this.dSUID, 0x02,
                                            this.percentToValue(val));
        }

        var angle = this.bladeAngleField.getValue();
        dSS.util.genericRequest.defer(500, this,
        [
            {
                url: '/json/device/setConfig',
                scope: this,
                params:
                {
                    'class': 64,
                    index: 0x04,
                    dsuid: this.dSUID,
                    value: this.percentToValue(angle, 0xff)
                },
                failureMsg: dSS._("Could not set device blade angle")
            }
        ]);
    },
    resetValues: function() {
        this.setDeviceValue(this.statusObject.outputValue);
        this.setBladeAngle(this.statusObject.bladeAngle);
    },
    resetDefaults: function() {
        this.statusObject.outputValue =
            this.percentToValue(this.valueField.getValue());
        this.statusObject.bladeAngle =
            this.percentToValue(this.bladeAngleField.getValue(), 0xff);
    },
    saveSettings: function() {
        var value = this.percentToValue(this.valueField.getValue());
        var angle = this.percentToValue(this.bladeAngleField.getValue(), 0xff);

        if ((this.statusObject.outputValue !== value) ||
            (this.statusObject.bladeAngle !== angle)) {
            if (this.matrix.shadebladeang !== true) {
                angle = undefined;
            }
            dSS.util.setSceneValue.defer(500, this,
                    [this.dSUID, this.scene, value, angle ]);
            this.resetDefaults();
        }
    }
});

Ext.reg('dssdevscenesettingspanel', dSS.devices.DeviceSceneSettingsPanel);

dSS.devices.DeviceSettingsWrapper = Ext.extend(Ext.Panel, {
    constructor: function(config) {

        var device = config.deviceRecord;

        var devCls = dSS.util.parseFunctionID(device.get('functionID'));
        var matrix = config.visibilityMatrix;

        var enabled = ((device.get('isPresent') === true) &&
                       (matrix.outvalue8 || matrix.shadeposition));

        this.scene = config.scene;

        this.resetButton = new Ext.Button({
            text: dSS._("Reset"),
            disabled: true,
            minWidth: 110,
            handler: this.resetSettings.createDelegate(this)
        });

        this.previewButton = new Ext.Button({
            text: dSS._("Preview"),
            minWidth: 110,
            disabled: !enabled,
            scope: this,
            handler: this.applySettings.createDelegate(this)
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formloaded', 'savebuttonstate', 'formsubmitted',
                            'formchanged' ],
            border: false,
            layout: 'border',
            height: 320,
            items:
            [
                {
                    xtype: 'dssdevscenesettingspanel',
                    ref: 'devsettings',
                    border: true,
                    bodyCssClass: 'x-panel-no-left-border x-panel-no-right-border x-panel-no-bottom-border',
                    dSUID: device.get('dsuid'),
                    DisplayID: device.get('DisplayID'),
                    devClass: devCls[0],
                    present: false,
                    matrix: matrix,
                    overrideHeight: 225,
                    extraWidth: 48,
                    region: 'north',
                    scene: config.scene,
                    dummy: !enabled
                },
                {
                    xtype: 'panel',
                    region: 'center',
                    height: 20,
                    border: false,
                    layout: 'hbox',
                    items:
                    [
                        {
                            xtype: 'box',
                            width: 10
                        },
                        this.resetButton,
                        {
                            xtype: 'box',
                            flex: 3
                        },
                        this.previewButton,
                        {
                            xtype: 'box',
                            width: 10
                        }
                    ]
                }
            ]
        });

        dSS.devices.DeviceSettingsWrapper.superclass.constructor.apply(this,
                                                                     arguments);

        this.on('formchanged', function() {
            this.previewButton.enable();
            this.resetButton.enable();
        },
        this);

        this.on('dataloaded', function(data) {
            if (data.loader !== dSS.devprops.Loaders.SCENE_VALUE) {
                return;
            }

            if (data.json.ok !== true) {
                Ext.Msg.alert(dSS._("Error"), data.json.message);
                this.fireEvent('formloaded', false);
                return;
            }

            this.devsettings.setDeviceValue(data.json.result.value);

            if (data.json.result.angle !== undefined) {
                this.devsettings.setBladeAngle(data.json.result.angle);
            }
            this.devsettings.resetDefaults();
            this.fireEvent('formloaded', true);
        },
        this);

        this.on('submit', function() {
            this.devsettings.saveSettings();
            this.resetButton.disable();
            this.fireEvent('formsubmitted');
        },
        this);

        dSS.MessageBus.on('deviceoutputmodechanged', function(checked) {
            // can't say why but using 'this' object produced strange Ext
            // errors, tried to disable items separately, i.e. via
            // this.devsettings - but 'devsettings' object got lost from time
            // to time, which I also can't explain.
            // The workaround below seems to do the trick.
            if (checked) {
                this.items.each(function(item) {
                    item.disable();
                });
            } else {
                this.items.each(function(item) {
                    item.enable();
                });
            }
        },
        this);
    },
    applySettings: function() {
        this.devsettings.writeValuesToDevice();
        this.previewButton.disable();
    },
    resetSettings: function() {
        this.devsettings.resetValues();
        this.previewButton.enable();
        this.resetButton.disable();
    }
});

Ext.reg('dssdevicesettingswrapper', dSS.devices.DeviceSettingsWrapper);

