Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID, group (configured group)
dSS.devprops.LocationForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.device === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing device"));
            return;
        }

        var list =
        [
            [
                dSS.config.Location.NONE,
                dSS._(dSS.config.ClusterLocationStrings[dSS.config.Location.NONE])
            ],
            [
                dSS.config.Location.NORTH,
                dSS._(dSS.config.ClusterLocationStrings[dSS.config.Location.NORTH])
            ],
            [
                dSS.config.Location.NORTH_EAST,
                dSS._(dSS.config.ClusterLocationStrings[dSS.config.Location.NORTH_EAST])
            ],
            [
                dSS.config.Location.EAST,
                dSS._(dSS.config.ClusterLocationStrings[dSS.config.Location.EAST])
            ],
            [
                dSS.config.Location.SOUTH_EAST,
                dSS._(dSS.config.ClusterLocationStrings[dSS.config.Location.SOUTH_EAST])
            ],
            [
                dSS.config.Location.SOUTH,
                dSS._(dSS.config.ClusterLocationStrings[dSS.config.Location.SOUTH])
            ],
            [
                dSS.config.Location.SOUTH_WEST,
                dSS._(dSS.config.ClusterLocationStrings[dSS.config.Location.SOUTH_WEST])
            ],
            [
                dSS.config.Location.WEST,
                dSS._(dSS.config.ClusterLocationStrings[dSS.config.Location.WEST])
            ],
            [
                dSS.config.Location.NORTH_WEST,
                dSS._(dSS.config.ClusterLocationStrings[dSS.config.Location.NORTH_WEST])
            ]
        ];

        this.device = config.device;
        this.dSUID = this.device.get('dsuid');
        this.direction = dSS.config.Location.NONE;

        this.addEvents('formsubmitted', 'formloaded');

        this.locationSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 145,
            mode: 'local',
            fieldLabel: dSS._("Orientation"),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data: list
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            height: 35,
            padding: 5,
            disabled: this.device.get('clusterLock'),
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.locationSelection
                    ]
                }
            ]
        });

        this.firstLoad = true;

        dSS.devprops.LocationForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.locationSelection.on('select', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.LOCATION) {
                    return;
                }

                if (data.json.ok === true) {
                    this.direction = data.json.result.direction;
                    this.locationSelection.setValue(this.direction);
                } else {
                    this.locationSelection.markInvalid(data.json.message);
                }


                this.fireEvent('formloaded', true);
            },
            this);


        this.on('submit', function() {
                var value = this.locationSelection.getValue();
                var dSUID = this.dSUID;

                dSS.util.genericRequest({
                    url: '/json/device/setCardinalDirection',
                    scope: this,
                    params:
                    {
                        dsuid: this.dSUID,
                        direction: value
                    },
                    failureMsg: dSS._("Could not set device orientation"),
                    evt: 'formsubmitted'
                });
            },
            this);
    }
});

Ext.reg('dssdevlocationform', dSS.devprops.LocationForm);

