Ext.namespace('dSS', 'dSS.grid', 'dSS.data', 'dSS.config', 'dSA',
              'dSS11.config');

Ext.override(Ext.ToolTip, {
    onTargetOver : function(e){
        if(this.disabled || e.within(this.target.dom, true)){
            return;
        }
        this.clearTimer('hide');
        this.triggerElement = e.getTarget();
        this.targetXY = e.getXY();
        this.delayShow();
    },

    onMouseMove : function(e){
        this.triggerElement = e.getTarget();
        this.targetXY = e.getXY();
        if(!this.hidden && this.trackMouse){
            this.setPagePosition(this.getTargetXY());
        }
    }
});

dSS.grid.ModulatorPanel = Ext.extend(Ext.grid.GridPanel, {
    initComponent: function() {
        var dSUIDrenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var shortId = record.get('DisplayID');
            if (record.get('isPresent') === true && record.get('isValid') === false) {
                shortId = dSS._('Loading') + ' ... ' + shortId;
            }
            return String.format('<div ext:qtip="dSUID: {0}">{1}</div>',
                                  record.get('dsuid'), shortId);
        };

        var deviceRegistrationRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var shortId = record.get('DisplayID');
            var str = "";
            if ((record.get('busMemberType') != dSS.config.BusMemberTypes.dSM11) &&
                (record.get('busMemberType') != dSS.config.BusMemberTypes.dSM12)) {
                return '';
            }
            if (record.get('ignoreActionsFromNewDevices') === true)
            {
                str = dSS._("enabled");
            }
            else
            {
                str = dSS._("disabled");
            }
            return str;
        };

        var ledCodeRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var code = record.get('ledCode');
            if (code < 0) {
                return "";
            }

            var cls = dSS.config.dSM_LED_CSS[code];
            if (cls === undefined) {
              return "";
            }

            metaData.css = cls;
            return String.format('<img class="padding-img" src="{0}"/>',
                                 Ext.BLANK_IMAGE_URL);
        };

        var firmwareRendererARM = function(value, metaData, record, rowIndex, colIndex, store) {
            if ((record.get('busMemberType') != dSS.config.BusMemberTypes.dSM11) &&
                (record.get('busMemberType') != dSS.config.BusMemberTypes.dSM12)) {
                return record.get('swVersion');
            }
            if ((dSS.config.dSMSystemFirmwareImageVersion.ARM.length > 0) &&
                (record.get('isPresent') === true) &&
                (record.get('isValid') === true) &&
                (dSS.config.dSMSystemFirmwareImageVersion.ARM !==
                    record.get('armSwVersion'))) {
                metaData.attr = 'style="color:red;"';
                return String.format('<div ext:qtip="{0}">{1}</div>',
                    dSS._("dSM firmware needs to be updated!"),
                                  record.get('armSwVersion'));
            }
            return record.get('armSwVersion');
        };

        var firmwareRendererDSP = function(value, metaData, record, rowIndex, colIndex, store) {
            if ((record.get('busMemberType') != dSS.config.BusMemberTypes.dSM11) &&
                (record.get('busMemberType') != dSS.config.BusMemberTypes.dSM12)) {
                return '';
            }
            if ((dSS.config.dSMSystemFirmwareImageVersion.DSP.length > 0) &&
                (record.get('isPresent') === true) &&
                (record.get('isValid') === true) &&
                (dSS.config.dSMSystemFirmwareImageVersion.DSP !==
                    record.get('dspSwVersion'))) {
                metaData.attr = 'style="color:red;"';
                return String.format('<div ext:qtip="{0}">{1}</div>',
                    dSS._("dSM firmware needs to be updated!"),
                                  record.get('dspSwVersion'));
            }
            return record.get('dspSwVersion');
        };

        var consumptionRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            if (record.get('hasMetering') != true) {
                return '';
            }
            return record.get('consumption');
        };

        var modulatorColumns = [];

        if (dSA.module === true) {
            modulatorColumns.push(
            {
                header: dSS._("Status"),
                tooltip: dSS._("Status"),
                width: 20,
                sortable: false,
                dataIndex: 'ledCode',
                id: 'ledCode',
                renderer: ledCodeRenderer
            });
        }

        modulatorColumns = modulatorColumns.concat([
        {
            id: 'name',
            header: dSS._("Name"),
            tooltip: dSS._("Name"),
            width: 80,
            sortable: true,
            dataIndex: 'name'
        },
        {
            id: 'dsuid',
            header: dSS._("dSM ID"),
            tooltip: dSS._("dSM ID"),
            width: 72,
            sortable: true,
            dataIndex: 'dsuid',
            renderer: dSUIDrenderer
        },
        {
            id: 'numdevices',
            header: dSS._("Number of devices"),
            tooltip: dSS._("Number of devices"),
            sortable: true,
            dataIndex: 'numDevices'
        },
        {
            id: 'deviceregistration',
            header: dSS._("Lock new devices"),
            tooltip: dSS._("Lock new devices"),
            sortable: true,
            dataIndex: 'ignoreActionsFromNewDevices',
            renderer: deviceRegistrationRenderer
        },
        {
            id: 'consumption',
            header: dSS._("Power (W)"),
            tooltip: dSS._("Power (W)"),
            width: 30,
            sortable: true,
            dataIndex: 'consumption',
            renderer: consumptionRenderer
        },
        {
            id: 'armSwVersion',
            header: dSS._("ARM software version"),
            tooltip: dSS._("ARM software version"),
            width: 30,
            sortable: true,
            dataIndex: 'armSwVersion',
            renderer: firmwareRendererARM
        },
        {
            id: 'dspSwVersion',
            header: dSS._("DSP software version"),
            tooltip: dSS._("DSP software version"),
            width: 30,
            sortable: true,
            dataIndex: 'dspSwVersion',
            renderer: firmwareRendererDSP
        },
        {
            id: 'hwVersion',
            header: dSS._("HW version"),
            tooltip: dSS._("Hardware version"),
            width: 30,
            sortable: true,
            dataIndex: 'hwVersionString'
        },
        {
            id: 'hardwareInfo',
            header: dSS._("HW info"),
            tooltip: dSS._("Hardware info"),
            width: 50,
            sortable: true,
            dataIndex: 'hwName'
        }
        ]);

        var rowSelect = new Ext.grid.RowSelectionModel({
            singleSelect: false
        });

        this.heightConfigured = false;
        this.selectionBlocked = true;

        Ext.apply(this, {
            store: dSS.data.Loader.getModulatorStore(),
            columns: modulatorColumns,
            enableDragDrop: false,
            stripeRows: true,
            title: dSS._("Meters"),
            selModel: rowSelect,
            viewConfig: {
                forceFit: true,
                getRowClass: function(record, index) {
                    var c = record.get('isPresent');
                    if (c === false) {
                        return 'nonPresentDevice';
                    }
                    return '';
                }
            }
        });

        this.tbar = this.buildTopToolbar();

        dSS.grid.ModulatorPanel.superclass.initComponent.apply(this, arguments);

        this.on('afterrender', function() {
            if (this.heightConfigured === false) {
                this.heightConfigured = true;
                this.setHeight(this.findParentByType('dsmoverviewpanel').getInnerHeight() / 2);
            }

        });

        this.on('viewready', function() {
            this.selectionBlocked = false;
            this.setInitialSelection();
        });

        dSS.MessageBus.on('structureloaded', function() {
            this.setInitialSelection();
        }, this);

        dSS.MessageBus.on('updatedsmdignoredevflag', function(dSUID, flag) {
            var mrec = this.getStore().getById(dSUID);
            if (mrec !== undefined)
            {
                mrec.set('ignoreActionsFromNewDevices', flag);
                mrec.commit();
            }
        }, this);

        dSS.data.Loader.getDeviceStore().on('load', function() {
            this.setInitialSelection();
        }, this);

        this.on('rowcontextmenu', function(grid, rowIndex, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var sel = this.getSelectionModel();
            if (!sel.isSelected(rowIndex)) {
                sel.selectRow(rowIndex, false);
            }

            var showRemoveMenuItem = false;
            var showMeterSettings = false;

            var records = sel.getSelections();

            var i;
            for (i = 0; i < records.length; i++) {
                var record = records[i];

                // if at least one record can be removed - set the flag
                if (record.get('isPresent') === false) {
                    showRemoveMenuItem = true;
                    break;
                }
            }
            for (i = 0; i < records.length; i++) {
                var record = records[i];

                // at least one selection is a dSMxx
                if ((record.get('busMemberType') == dSS.config.BusMemberTypes.dSM11) ||
                    (record.get('busMemberType') == dSS.config.BusMemberTypes.dSM12)) {
                    showMeterSettings = true;
                    break;
                }
            }

            var menuItem;

            var blinkDisabled = true;
            if ((record.get('busMemberType') == dSS.config.BusMemberTypes.dSM11) ||
                (record.get('busMemberType') == dSS.config.BusMemberTypes.dSM12)) {
                blinkDisabled = false;
            }

            menuItem = new Ext.menu.Item({
                iconCls: 'blinkdSMIcon',
                text: dSS._("Blink dSM LED"),
                disabled: blinkDisabled,
                handler: this.blinkdSM.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                iconCls: 'transmissionQuality',
                text: dSS._("Test transmission quality"),
                handler: this.testTransmissionQuality.createDelegate(this, rowIndex, true),
                disabled: !showMeterSettings
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                iconCls: 'checkDevicePresenceIcon',
                text: dSS._("Check device presence"),
                handler: this.checkDevicePresence.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                iconCls: 'reregisterDevicesIcon',
                text: dSS._("Reregister all devices"),
                handler: this.restartRegistration.createDelegate(this, rowIndex,
                                                                 true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                iconCls: 'identifyDeviceToolbarIcon',
                text: dSS._("Identify device"),
                handler: this.identifyDevice.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                text: dSS._("dSM settings"),
                iconCls: 'dSMSettings',
                handler: this.dSMSettings.createDelegate(this, rowIndex, true),
                disabled: !showMeterSettings
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: dSS._("Rename dSM"),
                iconCls: 'editIcon',
                handler: this.renameMeter.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                iconCls: 'viewCommunicationLogIcon',
                text: dSS._("View communication log"),
                disabled: (record.get('busMemberType') ==
                                            dSS.config.BusMemberTypes.VDC),
                handler: this.viewCommLog.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                text: dSS._("Remove dS-Meter"),
                iconCls: 'removeDeviceToolbarIcon',
                handler: this.removeMeter.createDelegate(this, rowIndex, true),
                disabled: !showRemoveMenuItem
            });
            this.contextMenu.add(menuItem);

            var current = sel.getSelected();
            if (current) {
                var url = String(current.get('VdcConfigURL'));
                if (url.length > 0) {
                    this.contextMenu.add('-');
                    menuItem = new Ext.menu.Item({
                        text: dSS._("Open vdc interface"),
                        iconCls: 'openVdcInterface',
                        href: url,
                        hrefTarget: '_blank'
                    });
                    this.contextMenu.add(menuItem);
                }
            }

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);

        rowSelect.on('rowselect', function(selModel, rowIndex, dataRecord) {
            dSS.MessageBus.fireEvent('filter_devices_circuit', dataRecord);
            this.lastSelectedDSUID = dataRecord.get('dsuid');
            this.getTopToolbar().removeMeterButton.setDisabled(dataRecord.get("isPresent"));
            var canBlink = ((dataRecord.get('busMemberType') == dSS.config.BusMemberTypes.dSM11) ||
                            (dataRecord.get('busMemberType') == dSS.config.BusMemberTypes.dSM12));
            this.getTopToolbar().blinkMeterButton.setDisabled(!canBlink);
            this.getTopToolbar().viewCommLogButton.setDisabled(dataRecord.get('busMemberType') == dSS.config.BusMemberTypes.VDC);
        },
        this);

        this.on('render', function() {
            this.tip = new Ext.ToolTip({
                target: this.getView().mainBody,
                trackMouse: true,
                renderTo: document.body,
                autoHide: true,
                listeners: {
                    beforeshow: this.updateTipBody.createDelegate(this),
                    move: this.updateTipBody.createDelegate(this)
                }
            });
        },
        this);
    },
    updateTipBody: function(tip) {
        if ((Ext.DomQuery.is(tip.triggerElement,
                            'td.x-grid3-td-ledCode') !== true) &&
            (Ext.DomQuery.is(tip.triggerElement,
                             'div.x-grid3-col-ledCode') !== true)) {
            return false;
        }
        var rowIndex = this.getView().findRowIndex(tip.triggerElement);
        tip.update(this.getStore().getAt(rowIndex).get('ledStatus'));
    },
    buildTopToolbar: function() {
        return [
        {
            iconCls: 'blinkdSMIcon',
            scope: this,
            handler: this.blinkdSM,
            ref: 'blinkMeterButton',
            tooltip: dSS._("Blink dSM LED")
        },
        '-',
        {
            iconCls: 'transmissionQuality',
            scope: this,
            handler: this.testTransmissionQuality,
            tooltip: dSS._("Test transmission quality")
        },
        {
            iconCls: 'checkDevicePresenceIcon',
            scope: this,
            handler: this.checkDevicePresence,
            tooltip: dSS._("Check device presence")
        },
        {
            iconCls: 'reregisterDevicesIcon',
            scope: this,
            handler: this.restartRegistration,
            tooltip: dSS._("Reregister all devices")
        },
        {
            iconCls: 'identifyDeviceToolbarIcon',
            scope: this,
            handler: this.identifyDevice,
            tooltip: dSS._("Identify device")
        },
        '-',
        {
            iconCls: 'dSMSettings',
            scope: this,
            handler: this.dSMSettings,
            tooltip: dSS._("dSM settings")
        },
        {
            iconCls: 'editIcon',
            scope: this,
            handler: this.renameMeter,
            tooltip: dSS._("Rename dSM")
        },
        '->',
        {
            iconCls: 'viewDeviceUpdateReportIcon',
            scope: this,
            handler: this.viewDeviceUpdateTaskReport,
            tooltip: dSS._("View device update status report")
        },
        '-',
        {
            iconCls: 'viewCommunicationLogIcon',
            scope: this,
            handler: this.viewCommLog,
            ref: 'viewCommLogButton',
            tooltip: dSS._("View communication log")
        },
        '-',
        {
            iconCls: 'removeDeviceToolbarIcon',
            handler: this.removeMeter,
            ref: 'removeMeterButton',
            tooltip: dSS._("Remove dS-Meter"),
            scope: this
        }];
    },

    renameMeter: function(item, event, rowIndex) {
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No meter selected, please select a meter from the list!"));
            return;
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: sel,
            jumpMode: true,
            title: dSS._("Rename dSM"),
            featureConfig: dSS.config.FeatureConfig.CIRCUIT,
            callerScope: this,
            width: 350,
            dialogType: dSS.config.dialogTypes.RENAME_DIALOG,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();

    },
    setInitialSelection: function() {
        if (this.selectionBlocked === true) {
            return;
        }

        if (!this.ownerCt.isVisible()) {
            return;
        }

        if (this.getStore().getCount() === 0) {
            this.getStore().clearFilter();
            return;
        }

        var selModel = this.getSelectionModel();
        var record;
        var index;
        var selected = false;

        if (selModel.hasSelection()) {
            record = selModel.getSelected();
            index = this.getStore().indexOf(record);
            if (index != -1) {
                selected = true;
            } else {
                record = undefined;
            }
        } else if (this.lastSelectedDSUID !== undefined) {
            index = this.getStore().findExact('dsuid', this.lastSelectedDSUID);
            if (index != -1) {
                record = this.getStore().getAt(index);
                selected = true;
            } else {
                record = undefined;
            }
        }

        if (selected === false) {
            if (this.getView().getRow(0) !== undefined) {
                this.getView().focusRow(0);
                selModel.selectFirstRow();
                record = this.getStore().getAt(0);
                index = 0;
            }
        }

        if (record !== undefined) {
            this.lastSelectedDSUID = record.get('dsuid');
            if (selected === true) {
                this.getView().focusRow(index);
                selModel.selectRecords([record], false);
            }
        }
    },
    rescanCircuit: function(item, event, rowIndex) {
        var record = this.getStore().getAt(rowIndex);
        Ext.Ajax.request({
            url: '/json/circuit/rescan',
            disableCaching: true,
            method: "GET",
            params: {
                dsuid: record.get('dsuid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok === false) {
                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Rescan failed"));
                    }
                }
                catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Rescan failed") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Rescan failed"));
            }
        });
    },
    showCopyWindow: function() {
        var tsvwindow = new dSS.util.GridToSVWindow({
            gridPanel: this,
            windowWidth: 640,
            windowHeight: 120,
            separator: "\t"
        });
        tsvwindow.show(this);
    },

    removeMeter: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no meter selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No meter selected, please select a meter from the list!"));
            return;
        }
        records = sel.getSelections();

        var skippedPresent = false;
        var notPresent = [];

        var i;
        for (i = 0; i < records.length; i++) {
            var record = records[i];

            // do not issue the ajax request if we know that it will fail
            if (record.get('isPresent') === true) {
                skippedPresent = true;
                continue;
            }

            notPresent.push(record);
        }

        if (notPresent.length > 0) {
            this.deleteMeter(notPresent);
        } else {
            if (skippedPresent === true) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Present meters can not be removed!"));
            }
        }
    },
    deleteMeter: function(records) {
        var store = this.getStore();
        if (records.length < 1) {
            this.el.unmask();
            return;
        }

        var record = records.shift();

        this.el.mask(dSS._("Deleting meter") + ": " + record.get('dsuid'));
        Ext.Ajax.request({
            url: '/json/apartment/removeMeter',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                dsuid: record.get('dsuid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        store.remove(record);
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS.gt.strargs(dSS._("digitalSTROM Meter %1 was removed"),
                                    [record.get('DisplayID')]));

                        this.deleteMeter(records);
                    } else {
                        this.el.unmask();
                        Ext.MessageBox.alert(dSS._("Could not remove"),
                                             dSS._(jsonData.message));
                    }
                }
                catch(err) {
                    this.el.unmask();
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send remove meter command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                this.el.unmask();
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send remove meter command to dSS"));
            }
        });
    },
    removeInactiveMeters: function() {
        Ext.Ajax.request({
            url: '/json/apartment/removeInactiveMeters',
            disableCaching: true,
            method: "GET",
            scope: this,
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        dSS.data.Loader.load();
                    } else {
                        Ext.MessageBox.alert(dSS._("Could not remove inactive meters"), jsonData.message);
                    }
                }
                catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send remove inactive meters command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send remove inactive meters command to dSS"));
            }
        });
    },
    getMeter: function(rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that
            // there is no selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No meter selected, please select a meter from " +
                          "the list!"));
                return null;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        // this should never happen
        if (record === undefined) {
            Ext.Msg.alert(dSS._("Error"), dSS._("dSM not found"));
            return null;
        }

        return record;
    },
    getMeter4dSA: function(rowIndex, ignoreNotPresent, notPresentMsg) {
        if (dSA.module !== true) {
            Ext.Msg.alert(dSS._("Notice"),
                          dSS._("This feature requires presence of the " +
                                "digitalSTROM Assistant, the setup is only " +
                                "available on the dSS11 hardware"));
        } else {
            var record = this.getMeter(rowIndex);
            if (record !== null) {
                if ((ignoreNotPresent === false) &&
                    (record.get('isPresent') === false)) {
                    Ext.Msg.alert(dSS._("Error"), notPresentMsg);
                    return null;
                }
            }

            return record;
        }

        return null;
    },
    restartRegistration: function(item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false,
                                       dSS._("Can not restart " +
                                       "registration, dSM is not present!"));
        if (record !== null) {
            dSA.Reactor.fireEvent('reregisterdevices', record.get('dsuid'));
        }
    },
    identifyDevice: function() {
        if (dSA.module !== true) {
            Ext.Msg.alert(dSS._("Notice"),
                          dSS._("This feature requires presence of the " +
                                "digitalSTROM Assistant, the setup is only " +
                                "available on the dSS11 hardware"));
            return;
        }
        dSA.Reactor.fireEvent('identifydevice',
                             dSS.data.Loader.getDeviceStore(),
                             this.getStore(),
                             this.findParentByType(Ext.Viewport).el);
    },
    blinkdSM: function(item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false,
                                 dSS._("Could not blink, dSM is not present!"));

        if (record !== null) {
            if ((record.get('busMemberType') == dSS.config.BusMemberTypes.dSM11) ||
                (record.get('busMemberType') == dSS.config.BusMemberTypes.dSM12)) {
                dSA.Reactor.fireEvent('blinkdsmled', record.get('dsuid'));
            } else {
                Ext.Msg.alert(dSS._("Notice"),
                          dSS._("Blinking dSM LED is not supported by this " +
                                "device"));
            }
        }
    },
    testTransmissionQuality: function(item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false, dSS._("Can not start " +
                             "transmission quality test, dSM is not present!"));
        if (record !== null) {
            if ((record.get('busMemberType') == dSS.config.BusMemberTypes.dSM11) ||
                (record.get('busMemberType') == dSS.config.BusMemberTypes.dSM12)) {
                dSA.Reactor.fireEvent('checktransmissionquality',
                        record.get('DisplayID'), record.get('dsid'),
                        record.get('dsuid'), record.get('name'),
                        this.findParentByType(Ext.Viewport).el);
            }
        }
    },
    checkDevicePresence: function(item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false, dSS._("Can not start " +
                             "device presence test, dSM is not present!"));
        if (record !== null) {
            dSA.Reactor.fireEvent('checkdevicepresence',
                                  record.get('DisplayID'),
                                  record.get('dsuid'), record.get('name'),
                                  this.findParentByType(Ext.Viewport).el);
        }
    },
    // only the dSS11 UI is capable of showing logs, so we will send a message
    // with the appropriate dSUID and the dSS11 websetup will take care of the
    // rest
    viewCommLog: function(item, event, rowIndex) {
        if (dSS11.config.dSSIntegration !== true) {
            Ext.Msg.alert(dSS._("Notice"),
                          dSS._("This feature is only available in the " +
                                "setup on dSS11 hardware"));
        } else {
            var record = this.getMeter(rowIndex);
            if (record !== null) {
                dSS.MessageBus.fireEvent('showlog', record.get('dsuid'));
            }
        }
    },
    viewDeviceUpdateTaskReport: function() {
        if (dSS11.config.dSSIntegration !== true) {
            Ext.Msg.alert(dSS._("Notice"),
                          dSS._("This feature is only available in the " +
                                "setup on dSS11 hardware"));
        } else {
            dSA.Reactor.fireEvent('showreport',
                                  dSA.config.TaskReportRequest.TYPE,
                                  dSA.config.TaskTypes.DEVICE_UPDATE_TASK,
                                  dSS._("Device update status report"));
        }
    },
    dSMSettings: function(item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false,
                                 dSS._("dSM settings are not available " +
                                       "because the dSM is not present!"));
        if (record !== null) {
            if ((record.get('busMemberType') == dSS.config.BusMemberTypes.dSM11) ||
                (record.get('busMemberType') == dSS.config.BusMemberTypes.dSM12)) {
                dSA.Reactor.fireEvent('dsmsettings', record.get('DisplayID'),
                                      record.get('dsid'), record.get('dsuid'),
                                      record.get('name'),
                                      this.findParentByType(Ext.Viewport).el);
            }
        }
    }
});

Ext.reg('dssmodulatorpanel', dSS.grid.ModulatorPanel);

